import cv2
import numpy as np
import urllib.request

url = 'http://192.168.244.20/cam-mid.jpg'

cap = cv2.VideoCapture(url)
whT = 320
confThreshold = 0.5
nmsThreshold = 0.3
classesfile = 'coco.names'
classNames = []
with open(classesfile, 'rt') as f:
    classNames = f.read().rstrip('\n').split('\n')

modelConfig = 'yolov3.cfg'
modelWeights = 'yolov3.weights'
net = cv2.dnn.readNetFromDarknet(modelConfig, modelWeights)
net.setPreferableBackend(cv2.dnn.DNN_BACKEND_OPENCV)
net.setPreferableTarget(cv2.dnn.DNN_TARGET_CPU)

def findObject(outputs, im):
    hT, wT, cT = im.shape
    bbox = []
    classIds = []
    confs = []
    found_cat = False
    found_bird = False
    found_car = False
    found_scissors = False 
    found_person = False
    found_aeroplane = False
    found_bench = False
    found_dog = False
    found_fork = False
    found_laptop = False
    found_mouse = False
    found_keyboard = False
    found_cellphone = False
    found_book = False
    found_hotdog = False
    found_bottle = False

    for output in outputs:
        for det in output:
            scores = det[5:]
            classId = np.argmax(scores)
            confidence = scores[classId]
            if confidence > confThreshold:
                w, h = int(det[2] * wT), int(det[3] * hT)
                x, y = int((det[0] * wT) - w / 2), int((det[1] * hT) - h / 2)
                bbox.append([x, y, w, h])
                classIds.append(classId)
                confs.append(float(confidence))
    
    indices = cv2.dnn.NMSBoxes(bbox, confs, confThreshold, nmsThreshold)
    
    if len(indices) > 0:
        for i in indices.flatten():
            box = bbox[i]
            x, y, w, h = box[0], box[1], box[2], box[3]
            if classNames[classIds[i]] == 'bird':
                found_bird = True
            elif classNames[classIds[i]] == 'cat':
                found_cat = True
            elif classNames[classIds[i]] == 'car':
                found_car = True
            elif classNames[classIds[i]] == 'scissors':
                found_scissors = True
            elif classNames[classIds[i]] == 'person':
                found_person = True
            elif classNames[classIds[i]] == 'aeroplane':
                found_aeroplane = True
            elif classNames[classIds[i]] == 'bench':
                found_bench = True
            elif classNames[classIds[i]] == 'dog':
                found_dog = True
            elif classNames[classIds[i]] == 'fork':
                found_fork = True
            elif classNames[classIds[i]] == 'laptop':
                found_laptop = True
            elif classNames[classIds[i]] == 'mouse':
                found_mouse = True
            elif classNames[classIds[i]] == 'keyboard':
                found_keyboard = True
            elif classNames[classIds[i]] == 'cell phone':
                found_cellphone = True
            elif classNames[classIds[i]] == 'book':
                found_book = True
            elif classNames[classIds[i]] == 'hot dog':
                found_cat = True
            elif classNames[classIds[i]] == 'bottle':
                found_cat = True
                
            if classNames[classIds[i]] == 'bird':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Ptak'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('bird')
                print(found_bird)

            if classNames[classIds[i]] == 'cat':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Kot'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('cat')
                print(found_cat)

            if classNames[classIds[i]] == 'car':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Samochod'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('samochod')
                print(found_car)
            if classNames[classIds[i]] == 'scissors':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'nozyczki'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('nozyczki')
                print(found_scissors)
            if classNames[classIds[i]] == 'person':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Czlowiek'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('czlowiek')
                print(found_person)
            if classNames[classIds[i]] == 'aeroplane':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Samolot'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('samolot')
                print(found_aeroplane)
            if classNames[classIds[i]] == 'bench':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Lawka'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('lawka')
                print(found_bench)
            if classNames[classIds[i]] == 'dog':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Pies'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('pies')
                print(found_dog)
            if classNames[classIds[i]] == 'fork':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Widelec'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('widelec')
                print(found_fork)
            if classNames[classIds[i]] == 'laptop':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Laptop'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('laptop')
                print(found_laptop)
            if classNames[classIds[i]] == 'mouse':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Myszka'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('myszka')
                print(found_mouse)
            if classNames[classIds[i]] == 'keyboard':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Klawiatura'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('klawiatura')
                print(found_keyboard)
            if classNames[classIds[i]] == 'cell phone':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Telefon'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('telefon')
                print(found_cellphone)
            if classNames[classIds[i]] == 'book':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'Ksiazka'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('ksiazka')
                print(found_book)
            if classNames[classIds[i]] == 'hot dog':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{classNames[classIds[i]].upper()} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('hot dog')
                print(found_hotdog)
            if classNames[classIds[i]] == 'bottle':
                cv2.rectangle(im, (x, y), (x + w, y + h), (255, 0, 255), 2)
                cv2.putText(im, f'{'butelka'} {int(confs[i] * 100)}%', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)
                print('bottle')
                print(found_bottle)
                      
            if found_cat and found_bird:
                print('alert')

while True:
    img_resp = urllib.request.urlopen(url)
    imgnp = np.array(bytearray(img_resp.read()), dtype=np.uint8)
    im = cv2.imdecode(imgnp, -1)
    sucess, img = cap.read()
    blob = cv2.dnn.blobFromImage(im, 1 / 255, (whT, whT), [0, 0, 0], 1, crop=False)
    net.setInput(blob)
    layernames = net.getLayerNames()
    outputNames = [layernames[i - 1] for i in net.getUnconnectedOutLayers().flatten()]

    outputs = net.forward(outputNames)
    findObject(outputs, im)

    cv2.imshow('Image', im)
    cv2.waitKey(1)
